/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 * Copyright (C) 2007 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package info.petar.texinline;

public final class Armor {

  private static char[] b2a32 = new char[] {
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
    'i', 'j', 'k', 'm', 'n', 'p', 'q', 'r',
    's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '2', '3', '4', '5', '6', '7', '8', '9'
  };

  private static int[] a2b32 = new int[] {
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, 24, 25, 26, 27, 28, 29, 30, 31, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, -1, 11, 12, -1,
    13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  };

  private static int[] b2a32rem = { 0, 2, 4, 5, 7 };
  private static int[] a2b32rem = { 0, -1, 1, -1, 2, 3, -1, 4 };
  
  private static char calc_b2a32 (int i) {
    return b2a32 [i];
  }
  
  private static int calc_a2b32 (char c) {
    int i = (int) c; // supposedly converts char to ascii
    return a2b32[i];
  }
  
  // public
  
  public static int armored32Length (int bblen) {
    int rem = bblen % 5;
    return (bblen / 5) * 8 + b2a32rem[rem];
  }
  
  public static int armored32Length (byte[] bb) {
    return armored32Length (bb.length);
  }
  
  public static int deArmored32Length (int len) {
    if (len < 0)
      return -1;
    int rem = a2b32rem[len & 7];
    if (rem < 0)
      return -1;
    if (len == 0)
      return 0;
    return (len >> 3) * 5 + rem;
  }
  
  public static int deArmored32Length (char[] cc, int len) {
    if (len < 0)
      len = armor32Length (cc);
    return deArmored32Length (len);
  }
  
  public static char[] armor32 (byte[] bb) {
    char[] cc = new char[armored32Length (bb)];
    return armor32 (bb, cc, 0);
  }
  
  private static int prom (byte b) {
    int i = b;
    i &= 0xff;
    return i;
  }
  
  public static char[] armor32 (byte[] bb, char[] cc, int cc_off) {
    
    int p = 0;
    int rem = bb.length % 5;
    int e = p + (bb.length - rem);
    //char[] res = new char[(bb.length / 5) * 8 + b2a32rem [rem]];
    int d = cc_off;
    
    while (p < e) {
      cc[d + 0] = calc_b2a32 (prom (bb[p + 0]) >>> 3);
      cc[d + 1] = calc_b2a32 ((prom (bb[p + 0]) & 0x7) << 2 | 
                               prom (bb[p + 1]) >>> 6);
      cc[d + 2] = calc_b2a32 (prom (bb[p + 1]) >>> 1 & 0x1f);
      cc[d + 3] = calc_b2a32 ((prom (bb[p + 1]) & 0x1) << 4 | 
                              prom (bb[p + 2]) >>> 4);
      cc[d + 4] = calc_b2a32 ((prom (bb[p + 2]) & 0xf) << 1 | 
                              prom (bb[p + 3]) >>> 7);
      cc[d + 5] = calc_b2a32 (prom (bb[p + 3]) >>> 2 & 0x1f);
      cc[d + 6] = calc_b2a32 ((prom (bb[p + 3]) & 0x3) << 3 | 
                              prom (bb[p + 4]) >>> 5);
      cc[d + 7] = calc_b2a32 (prom (bb[p + 4]) & 0x1f);
      p += 5;
      d += 8;
    }
    
    switch (rem) {
    case 4:
      cc[d + 6] = calc_b2a32 ((prom (bb[p + 3]) & 0x3) << 3);
      cc[d + 5] = calc_b2a32 (prom (bb[p + 3]) >>> 2 & 0x1f);
      cc[d + 4] = calc_b2a32 ((prom (bb[p + 2]) & 0xf) << 1 | 
                              prom (bb[p + 3]) >>> 7);
      cc[d + 3] = calc_b2a32 ((prom (bb[p + 1]) & 0x1) << 4 | 
                              prom (bb[p + 2]) >>> 4);
      cc[d + 2] = calc_b2a32 (prom (bb[p + 1]) >>> 1 & 0x1f);
      cc[d + 1] = calc_b2a32 ((prom (bb[p + 0]) & 0x7) << 2 | 
                              prom (bb[p + 1]) >>> 6);
      cc[d + 0] = calc_b2a32 (prom (bb[p + 0]) >>> 3);
      d += 7;
      break;
    case 3:
      cc[d + 4] = calc_b2a32 ((prom (bb[p + 2]) & 0xf) << 1);
      cc[d + 3] = calc_b2a32 ((prom (bb[p + 1]) & 0x1) << 4 | 
                              prom (bb[p + 2]) >>> 4);
      cc[d + 2] = calc_b2a32 (prom (bb[p + 1]) >>> 1 & 0x1f);
      cc[d + 1] = calc_b2a32 ((prom (bb[p + 0]) & 0x7) << 2 | 
                              prom (bb[p + 1]) >>> 6);
      cc[d + 0] = calc_b2a32 (prom (bb[p + 0]) >>> 3);
      d += 5;
      break;
    case 2:
      cc[d + 3] = calc_b2a32 ((prom (bb[p + 1]) & 0x1) << 4);
      cc[d + 2] = calc_b2a32 (prom (bb[p + 1]) >>> 1 & 0x1f);
      cc[d + 1] = calc_b2a32 ((prom (bb[p + 0]) & 0x7) << 2 | 
                              prom (bb[p + 1]) >>> 6);
      cc[d + 0] = calc_b2a32 (prom (bb[p + 0]) >>> 3);
      d += 4;
      break;
    case 1:
      cc[d + 1] = calc_b2a32 ((prom (bb[p + 0]) & 0x7) << 2);
      cc[d + 0] = calc_b2a32 (prom (bb[p + 0]) >>> 3);
      d += 2;
      break;
    }
    
    assert (d == cc_off + armored32Length (bb));
    return cc;
  }
  
  public static int armor32Length (char[] cc) {
    int p = 0;
    while (p < cc.length && calc_a2b32 (cc[p]) >= 0)
      p++;
    return p;
  }
  
  public static byte[] deArmor32 (char[] cc, int len) {
    byte[] bb = new byte[deArmored32Length (cc, len)];
    return deArmor32 (cc, len, bb, 0);
  }
  
  public static byte[] deArmor32 (char[] cc, int len, byte[] bb, int bb_off) {
    
    assert false; // function not tested

    int s = 0;
    
    if (len < 0)
      len = armor32Length (cc);
    int rem = a2b32rem [len & 7];
    if (rem < 0)
      return null;
    if (len == 0)
      return new byte [0];
    
    int d = bb_off;
    
    int c0, c1, c2, c3, c4, c5, c6, c7;
    
    for (int e = len & ~7; s < e; s += 8, d+= 5) {
      c0 = calc_a2b32 (cc[s + 0]);
      c1 = calc_a2b32 (cc[s + 1]);
      bb[d + 0] = (byte) (c0 << 3 | c1 >> 2);
      c2 = calc_a2b32 (cc[s + 2]);
      c3 = calc_a2b32 (cc[s + 3]);
      bb[d + 1] = (byte) (c1 << 6 | c2 << 1 | c3 >> 4);
      c4 = calc_a2b32 (cc[s + 4]);
      bb[d + 2] = (byte) (c3 << 4 | c4 >> 1);
      c5 = calc_a2b32 (cc[s + 5]);
      c6 = calc_a2b32 (cc[s + 6]);
      bb[d + 3] = (byte) (c4 << 7 | c5 << 2 | c6 >> 3);
      c7 = calc_a2b32 (cc[s + 7]);
      bb[d + 4] = (byte) (c6 << 5 | c7);
    }
    
    if (rem >= 1) {
      c0 = calc_a2b32 (cc[s + 0]);
      c1 = calc_a2b32 (cc[s + 1]);
      bb[d++] = (byte) (c0 << 3 | c1 >> 2);
      if (rem >= 2) {
        c2 = calc_a2b32 (cc[s + 2]);
        c3 = calc_a2b32 (cc[s + 3]);
        bb[d++] = (byte) (c1 << 6 | c2 << 1 | c3 >> 4);
        if (rem >= 3) {
          c4 = calc_a2b32 (cc[s + 4]);
          bb[d++] = (byte) (c3 << 4 | c4 >> 1);
          if (rem >= 4) {
            c5 = calc_a2b32 (cc[s + 5]);
            c6 = calc_a2b32 (cc[s + 6]);
            bb[d++] = (byte) (c4 << 7 | c5 << 2 | c6 >> 3);
          }
        }
      }
    }
    
    assert (d == bb_off + deArmored32Length (cc, len));
    return bb;
  }
  
  public static byte[] deArmor32 (char[] cc) { 
    return deArmor32 (cc, -1); 
  }
  
  public static String printableBytes (byte[] bb) {
    return new String (armor32 (bb));
  }
  
} // Armor
