/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package info.petar.texinline;

public final class Sgml {
  
  public static String unescapeSingle (String s) {
    if (s == null)
      return null;
    StringBuilder sb = new StringBuilder ();
    for (int i = 0; i < s.length (); i++) 
      if (s.charAt (i) == '\\') {
        i++;
        if (i >= s.length ()) {
          System.err.println ("String literal has " +
                              "unfinished escape sequence!");
          System.exit (1);
        }
        switch (s.charAt (i)) {
        case '\'':
          sb.append ('\'');
          break ;
        case '\\':
          sb.append ('\\');
          break ;
        default:
          System.err.println ("String literal has invalid escape sequence!");
          System.exit (1);
        }
      } else
        sb.append (s.charAt (i));
    return sb.toString ();
  }
  
  public static String unescapeDouble (String s) {
    if (s == null)
      return null;
    StringBuilder sb = new StringBuilder ();
    for (int i = 0; i < s.length (); i++) 
      if (s.charAt (i) == '\\') {
        i++;
        if (i >= s.length ()) {
          System.err.println ("String literal has " +
                              "unfinished escape sequence!");
          System.exit (1);
        }
        switch (s.charAt (i)) {
        case '"':
          sb.append ('"');
          break ;
        case '\\':
          sb.append ('\\');
          break ;
        default:
          System.err.println ("String literal has invalid escape sequence!");
          System.exit (1);
        }
      } else
        sb.append (s.charAt (i));
    return sb.toString ();
  }
  
  // escapes a string for double-quotes form
  public static String escapeDouble (String s) {
    if (s == null)
      return null;
    StringBuilder sb = new StringBuilder ();
    
    for (int i = 0; i < s.length (); i++) {
      char c = s.charAt (i);
      switch (c) {
      case '\\':
        sb.append ("\\\\");
        break ;
      case '"':
        sb.append ("\\\"");
        break ;
      default:
        sb.append (c);
      }
    }
    return sb.toString ();
  }
  
} // Sgml
