/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package info.petar.texinline;

import java.util.HashMap;
import java.util.Map;
import xtc.tree.Attribute;
import xtc.tree.AttributeList;
import xtc.tree.Node;
import xtc.tree.Node;

public final class Tag extends Node {
  
  public enum Type { Begin, End, BeginEnd }

  private String name;
  private Type type;
  private HashMap<String, String> attribs;
  
  public String getName () { 
    return name;
  }

  public Type getType () {
    return type;
  }

  public HashMap<String, String> getAttribs () {
    return attribs;
  }

  public String getAttrib (String key) {
    return attribs.get (key);
  }
  
  //

  public Tag (Type t, Node n) {
    super ();
    type = t;
    attribs = new HashMap<String, String> ();
    name = (String) n.get (0);
    for (int i = 1; i < n.size (); i++) {
      Node a = (Node) n.get (i);
      if (a.size () == 1)
        attribs.put ((String) a.get (0), null);
      else
        attribs.put ((String) a.get (0), (String) a.get (1));
    }
  }

  public String source () {
    StringBuilder sb = new StringBuilder ();
    sb.append ("<");
    if (type == Type.End)
      sb.append ("/");
    sb.append (getName ());
    
    for (Map.Entry<String,String> kv : attribs.entrySet ())
      sb.append (" " + kv.getKey () + 
                 "=\"" + 
                 Sgml.escapeDouble (kv.getValue ()) +
                 "\""
                 );

    if (type == Type.BeginEnd)
      sb.append ("/");
    sb.append (">");
    return sb.toString ();
  }

} // Tag
